@@ -1,9 +1,10 @@
 """Test module for KeycloakOpenID."""
 
 from inspect import iscoroutinefunction, signature
-from typing import Tuple
 from unittest import mock
 
+import jwcrypto.jwk
+import jwcrypto.jws
 import pytest
 
 from keycloak import KeycloakAdmin, KeycloakOpenID
@@ -20,16 +21,18 @@ from keycloak.exceptions import (
     KeycloakPostError,
     KeycloakRPTNotFound,
 )
+from tests.conftest import KeycloakTestEnv
 
 
-def test_keycloak_openid_init(env):
-    """Test KeycloakOpenId's init method.
+def test_keycloak_openid_init(env: KeycloakTestEnv) -> None:
+    """
+    Test KeycloakOpenId's init method.
 
     :param env: Environment fixture
     :type env: KeycloakTestEnv
     """
     oid = KeycloakOpenID(
-        server_url=f"http://{env.KEYCLOAK_HOST}:{env.KEYCLOAK_PORT}",
+        server_url=f"http://{env.keycloak_host}:{env.keycloak_port}",
         realm_name="master",
         client_id="admin-cli",
     )
@@ -41,15 +44,16 @@ def test_keycloak_openid_init(env):
     assert isinstance(oid.authorization, Authorization)
 
 
-def test_well_known(oid: KeycloakOpenID):
-    """Test the well_known method.
+def test_well_known(oid: KeycloakOpenID) -> None:
+    """
+    Test the well_known method.
 
     :param oid: Keycloak OpenID client
     :type oid: KeycloakOpenID
     """
     res = oid.well_known()
     assert res is not None
-    assert res != dict()
+    assert res != {}
     for key in [
         "acr_values_supported",
         "authorization_encryption_alg_values_supported",
@@ -108,8 +112,9 @@ def test_well_known(oid: KeycloakOpenID):
         assert key in res
 
 
-def test_auth_url(env, oid: KeycloakOpenID):
-    """Test the auth_url method.
+def test_auth_url(env: KeycloakTestEnv, oid: KeycloakOpenID) -> None:
+    """
+    Test the auth_url method.
 
     :param env: Environment fixture
     :type env: KeycloakTestEnv
@@ -118,15 +123,15 @@ def test_auth_url(env, oid: KeycloakOpenID):
     """
     res = oid.auth_url(redirect_uri="http://test.test/*")
     assert (
-        res
-        == f"http://{env.KEYCLOAK_HOST}:{env.KEYCLOAK_PORT}/realms/{oid.realm_name}"
-        + f"/protocol/openid-connect/auth?client_id={oid.client_id}&response_type=code"
-        + "&redirect_uri=http://test.test/*&scope=email&state="
+        res == f"http://{env.keycloak_host}:{env.keycloak_port}/realms/{oid.realm_name}"
+        f"/protocol/openid-connect/auth?client_id={oid.client_id}&response_type=code"
+        "&redirect_uri=http://test.test/*&scope=email&state=&nonce="
     )
 
 
-def test_token(oid_with_credentials: Tuple[KeycloakOpenID, str, str]):
-    """Test the token method.
+def test_token(oid_with_credentials: tuple[KeycloakOpenID, str, str]) -> None:
+    """
+    Test the token method.
 
     :param oid_with_credentials: Keycloak OpenID client with pre-configured user credentials
     :type oid_with_credentials: Tuple[KeycloakOpenID, str, str]
@@ -175,9 +180,11 @@ def test_token(oid_with_credentials: Tuple[KeycloakOpenID, str, str]):
 
 
 def test_exchange_token(
-    oid_with_credentials: Tuple[KeycloakOpenID, str, str], admin: KeycloakAdmin
-):
-    """Test the exchange token method.
+    oid_with_credentials: tuple[KeycloakOpenID, str, str],
+    admin: KeycloakAdmin,
+) -> None:
+    """
+    Test the exchange token method.
 
     :param oid_with_credentials: Keycloak OpenID client with pre-configured user credentials
     :type oid_with_credentials: Tuple[KeycloakOpenID, str, str]
@@ -196,7 +203,7 @@ def test_exchange_token(
             admin.get_client_role(
                 client_id=admin.get_client_id(client_id="realm-management"),
                 role_name="impersonation",
-            )
+            ),
         ],
     )
 
@@ -213,7 +220,9 @@ def test_exchange_token(
 
     # Exchange token with the new user
     new_token = oid.exchange_token(
-        token=token["access_token"], audience=oid.client_id, subject=username
+        token=token["access_token"],
+        audience=oid.client_id,
+        subject=username,
     )
     assert oid.userinfo(token=new_token["access_token"]) == {
         "email": f"{username}@test.test",
@@ -227,8 +236,9 @@ def test_exchange_token(
     assert token != new_token
 
 
-def test_logout(oid_with_credentials):
-    """Test logout.
+def test_logout(oid_with_credentials: tuple[KeycloakOpenID, str, str]) -> None:
+    """
+    Test logout.
 
     :param oid_with_credentials: Keycloak OpenID client with pre-configured user credentials
     :type oid_with_credentials: Tuple[KeycloakOpenID, str, str]
@@ -236,15 +246,16 @@ def test_logout(oid_with_credentials):
     oid, username, password = oid_with_credentials
 
     token = oid.token(username=username, password=password)
-    assert oid.userinfo(token=token["access_token"]) != dict()
-    assert oid.logout(refresh_token=token["refresh_token"]) == dict()
+    assert oid.userinfo(token=token["access_token"]) != {}
+    assert oid.logout(refresh_token=token["refresh_token"]) == {}
 
     with pytest.raises(KeycloakAuthenticationError):
         oid.userinfo(token=token["access_token"])
 
 
-def test_certs(oid: KeycloakOpenID):
-    """Test certificates.
+def test_certs(oid: KeycloakOpenID) -> None:
+    """
+    Test certificates.
 
     :param oid: Keycloak OpenID client
     :type oid: KeycloakOpenID
@@ -252,8 +263,9 @@ def test_certs(oid: KeycloakOpenID):
     assert len(oid.certs()["keys"]) == 2
 
 
-def test_public_key(oid: KeycloakOpenID):
-    """Test public key.
+def test_public_key(oid: KeycloakOpenID) -> None:
+    """
+    Test public key.
 
     :param oid: Keycloak OpenID client
     :type oid: KeycloakOpenID
@@ -262,9 +274,11 @@ def test_public_key(oid: KeycloakOpenID):
 
 
 def test_entitlement(
-    oid_with_credentials_authz: Tuple[KeycloakOpenID, str, str], admin: KeycloakAdmin
-):
-    """Test entitlement.
+    oid_with_credentials_authz: tuple[KeycloakOpenID, str, str],
+    admin: KeycloakAdmin,
+) -> None:
+    """
+    Test entitlement.
 
     :param oid_with_credentials_authz: Keycloak OpenID client configured as an authorization
         server with client credentials
@@ -275,15 +289,16 @@ def test_entitlement(
     oid, username, password = oid_with_credentials_authz
     token = oid.token(username=username, password=password)
     resource_server_id = admin.get_client_authz_resources(
-        client_id=admin.get_client_id(oid.client_id)
+        client_id=admin.get_client_id(oid.client_id),
     )[0]["_id"]
 
     with pytest.raises(KeycloakDeprecationError):
         oid.entitlement(token=token["access_token"], resource_server_id=resource_server_id)
 
 
-def test_introspect(oid_with_credentials: Tuple[KeycloakOpenID, str, str]):
-    """Test introspect.
+def test_introspect(oid_with_credentials: tuple[KeycloakOpenID, str, str]) -> None:
+    """
+    Test introspect.
 
     :param oid_with_credentials: Keycloak OpenID client with pre-configured user credentials
     :type oid_with_credentials: Tuple[KeycloakOpenID, str, str]
@@ -293,15 +308,18 @@ def test_introspect(oid_with_credentials: Tuple[KeycloakOpenID, str, str]):
 
     assert oid.introspect(token=token["access_token"])["active"]
     assert oid.introspect(
-        token=token["access_token"], rpt="some", token_type_hint="requesting_party_token"
+        token=token["access_token"],
+        rpt="some",
+        token_type_hint="requesting_party_token",  # noqa: S106
     ) == {"active": False}
 
     with pytest.raises(KeycloakRPTNotFound):
-        oid.introspect(token=token["access_token"], token_type_hint="requesting_party_token")
+        oid.introspect(token=token["access_token"], token_type_hint="requesting_party_token")  # noqa: S106
 
 
-def test_decode_token(oid_with_credentials: Tuple[KeycloakOpenID, str, str]):
-    """Test decode token.
+def test_decode_token(oid_with_credentials: tuple[KeycloakOpenID, str, str]) -> None:
+    """
+    Test decode token.
 
     :param oid_with_credentials: Keycloak OpenID client with pre-configured user credentials
     :type oid_with_credentials: Tuple[KeycloakOpenID, str, str]
@@ -317,8 +335,49 @@ def test_decode_token(oid_with_credentials: Tuple[KeycloakOpenID, str, str]):
     assert decoded_refresh_token["typ"] == "Refresh", decoded_refresh_token
 
 
-def test_load_authorization_config(oid_with_credentials_authz: Tuple[KeycloakOpenID, str, str]):
-    """Test load authorization config.
+def test_decode_token_invalid_token(oid_with_credentials: tuple[KeycloakOpenID, str, str]) -> None:
+    """
+    Test decode token with an invalid token.
+
+    :param oid_with_credentials: Keycloak OpenID client with pre-configured user credentials
+    :type oid_with_credentials: Tuple[KeycloakOpenID, str, str]
+    """
+    oid, username, password = oid_with_credentials
+    token = oid.token(username=username, password=password)
+    access_token = token["access_token"]
+    decoded_access_token = oid.decode_token(token=access_token)
+
+    key = oid.public_key()
+    key = "-----BEGIN PUBLIC KEY-----\n" + key + "\n-----END PUBLIC KEY-----"
+    key = jwcrypto.jwk.JWK.from_pem(key.encode("utf-8"))
+
+    invalid_access_token = access_token + "a"
+    with pytest.raises(jwcrypto.jws.InvalidJWSSignature):
+        decoded_invalid_access_token = oid.decode_token(token=invalid_access_token, validate=True)
+
+    with pytest.raises(jwcrypto.jws.InvalidJWSSignature):
+        decoded_invalid_access_token = oid.decode_token(
+            token=invalid_access_token,
+            validate=True,
+            key=key,
+        )
+
+    decoded_invalid_access_token = oid.decode_token(token=invalid_access_token, validate=False)
+    assert decoded_access_token == decoded_invalid_access_token
+
+    decoded_invalid_access_token = oid.decode_token(
+        token=invalid_access_token,
+        validate=False,
+        key=key,
+    )
+    assert decoded_access_token == decoded_invalid_access_token
+
+
+def test_load_authorization_config(
+    oid_with_credentials_authz: tuple[KeycloakOpenID, str, str],
+) -> None:
+    """
+    Test load authorization config.
 
     :param oid_with_credentials_authz: Keycloak OpenID client configured as an authorization
         server with client credentials
@@ -333,12 +392,14 @@ def test_load_authorization_config(oid_with_credentials_authz: Tuple[KeycloakOpe
     assert isinstance(oid.authorization.policies["test-authz-rb-policy"].roles[0], Role)
     assert len(oid.authorization.policies["test-authz-rb-policy"].permissions) == 2
     assert isinstance(
-        oid.authorization.policies["test-authz-rb-policy"].permissions[0], Permission
+        oid.authorization.policies["test-authz-rb-policy"].permissions[0],
+        Permission,
     )
 
 
-def test_get_policies(oid_with_credentials_authz: Tuple[KeycloakOpenID, str, str]):
-    """Test get policies.
+def test_get_policies(oid_with_credentials_authz: tuple[KeycloakOpenID, str, str]) -> None:
+    """
+    Test get policies.
 
     :param oid_with_credentials_authz: Keycloak OpenID client configured as an authorization
         server with client credentials
@@ -355,15 +416,17 @@ def test_get_policies(oid_with_credentials_authz: Tuple[KeycloakOpenID, str, str
 
     orig_client_id = oid.client_id
     oid.client_id = "account"
-    assert oid.get_policies(token=token["access_token"], method_token_info="decode") == []
+    assert oid.get_policies(token=token["access_token"], method_token_info="decode") == []  # noqa: S106
     policy = Policy(name="test", type="role", logic="POSITIVE", decision_strategy="UNANIMOUS")
     policy.add_role(role="account/view-profile")
     oid.authorization.policies["test"] = policy
     assert [
-        str(x) for x in oid.get_policies(token=token["access_token"], method_token_info="decode")
+        str(x)
+        for x in oid.get_policies(token=token["access_token"], method_token_info="decode")  # noqa: S106
     ] == ["Policy: test (role)"]
     assert [
-        repr(x) for x in oid.get_policies(token=token["access_token"], method_token_info="decode")
+        repr(x)
+        for x in oid.get_policies(token=token["access_token"], method_token_info="decode")  # noqa: S106
     ] == ["<Policy: test (role)>"]
     oid.client_id = orig_client_id
 
@@ -372,8 +435,9 @@ def test_get_policies(oid_with_credentials_authz: Tuple[KeycloakOpenID, str, str
         oid.get_policies(token=token["access_token"])
 
 
-def test_get_permissions(oid_with_credentials_authz: Tuple[KeycloakOpenID, str, str]):
-    """Test get policies.
+def test_get_permissions(oid_with_credentials_authz: tuple[KeycloakOpenID, str, str]) -> None:
+    """
+    Test get policies.
 
     :param oid_with_credentials_authz: Keycloak OpenID client configured as an authorization
         server with client credentials
@@ -390,22 +454,25 @@ def test_get_permissions(oid_with_credentials_authz: Tuple[KeycloakOpenID, str,
 
     orig_client_id = oid.client_id
     oid.client_id = "account"
-    assert oid.get_permissions(token=token["access_token"], method_token_info="decode") == []
+    assert oid.get_permissions(token=token["access_token"], method_token_info="decode") == []  # noqa: S106
     policy = Policy(name="test", type="role", logic="POSITIVE", decision_strategy="UNANIMOUS")
     policy.add_role(role="account/view-profile")
     policy.add_permission(
         permission=Permission(
-            name="test-perm", type="resource", logic="POSITIVE", decision_strategy="UNANIMOUS"
-        )
+            name="test-perm",
+            type="resource",
+            logic="POSITIVE",
+            decision_strategy="UNANIMOUS",
+        ),
     )
     oid.authorization.policies["test"] = policy
     assert [
         str(x)
-        for x in oid.get_permissions(token=token["access_token"], method_token_info="decode")
+        for x in oid.get_permissions(token=token["access_token"], method_token_info="decode")  # noqa: S106
     ] == ["Permission: test-perm (resource)"]
     assert [
         repr(x)
-        for x in oid.get_permissions(token=token["access_token"], method_token_info="decode")
+        for x in oid.get_permissions(token=token["access_token"], method_token_info="decode")  # noqa: S106
     ] == ["<Permission: test-perm (resource)>"]
     oid.client_id = orig_client_id
 
@@ -414,8 +481,9 @@ def test_get_permissions(oid_with_credentials_authz: Tuple[KeycloakOpenID, str,
         oid.get_permissions(token=token["access_token"])
 
 
-def test_uma_permissions(oid_with_credentials_authz: Tuple[KeycloakOpenID, str, str]):
-    """Test UMA permissions.
+def test_uma_permissions(oid_with_credentials_authz: tuple[KeycloakOpenID, str, str]) -> None:
+    """
+    Test UMA permissions.
 
     :param oid_with_credentials_authz: Keycloak OpenID client configured as an authorization
         server with client credentials
@@ -429,9 +497,11 @@ def test_uma_permissions(oid_with_credentials_authz: Tuple[KeycloakOpenID, str,
 
 
 def test_has_uma_access(
-    oid_with_credentials_authz: Tuple[KeycloakOpenID, str, str], admin: KeycloakAdmin
-):
-    """Test has UMA access.
+    oid_with_credentials_authz: tuple[KeycloakOpenID, str, str],
+    admin: KeycloakAdmin,
+) -> None:
+    """
+    Test has UMA access.
 
     :param oid_with_credentials_authz: Keycloak OpenID client configured as an authorization
         server with client credentials
@@ -462,16 +532,18 @@ def test_has_uma_access(
     assert (
         str(
             oid.has_uma_access(
-                token=admin.connection.token["access_token"], permissions="Default Resource"
-            )
+                token=admin.connection.token["access_token"],
+                permissions="Default Resource",
+            ),
         )
         == "AuthStatus(is_authorized=False, is_logged_in=False, missing_permissions="
-        + "{'Default Resource'})"
+        "{'Default Resource'})"
     )
 
 
-def test_device(oid_with_credentials_device: Tuple[KeycloakOpenID, str, str]):
-    """Test device authorization flow.
+def test_device(oid_with_credentials_device: tuple[KeycloakOpenID, str, str]) -> None:
+    """
+    Test device authorization flow.
 
     :param oid_with_credentials_device: Keycloak OpenID client with pre-configured user
         credentials and device authorization flow enabled
@@ -484,7 +556,7 @@ def test_device(oid_with_credentials_device: Tuple[KeycloakOpenID, str, str]):
         "user_code": mock.ANY,
         "verification_uri": f"http://localhost:8081/realms/{oid.realm_name}/device",
         "verification_uri_complete": f"http://localhost:8081/realms/{oid.realm_name}/"
-        + f"device?user_code={res['user_code']}",
+        f"device?user_code={res['user_code']}",
         "expires_in": 600,
         "interval": 5,
     }
@@ -494,15 +566,16 @@ def test_device(oid_with_credentials_device: Tuple[KeycloakOpenID, str, str]):
 
 
 @pytest.mark.asyncio
-async def test_a_well_known(oid: KeycloakOpenID):
-    """Test the well_known method.
+async def test_a_well_known(oid: KeycloakOpenID) -> None:
+    """
+    Test the well_known method.
 
     :param oid: Keycloak OpenID client
     :type oid: KeycloakOpenID
     """
     res = await oid.a_well_known()
     assert res is not None
-    assert res != dict()
+    assert res != {}
     for key in [
         "acr_values_supported",
         "authorization_encryption_alg_values_supported",
@@ -562,8 +635,9 @@ async def test_a_well_known(oid: KeycloakOpenID):
 
 
 @pytest.mark.asyncio
-async def test_a_auth_url(env, oid: KeycloakOpenID):
-    """Test the auth_url method.
+async def test_a_auth_url(env: KeycloakTestEnv, oid: KeycloakOpenID) -> None:
+    """
+    Test the auth_url method.
 
     :param env: Environment fixture
     :type env: KeycloakTestEnv
@@ -572,16 +646,16 @@ async def test_a_auth_url(env, oid: KeycloakOpenID):
     """
     res = await oid.a_auth_url(redirect_uri="http://test.test/*")
     assert (
-        res
-        == f"http://{env.KEYCLOAK_HOST}:{env.KEYCLOAK_PORT}/realms/{oid.realm_name}"
-        + f"/protocol/openid-connect/auth?client_id={oid.client_id}&response_type=code"
-        + "&redirect_uri=http://test.test/*&scope=email&state="
+        res == f"http://{env.keycloak_host}:{env.keycloak_port}/realms/{oid.realm_name}"
+        f"/protocol/openid-connect/auth?client_id={oid.client_id}&response_type=code"
+        "&redirect_uri=http://test.test/*&scope=email&state=&nonce="
     )
 
 
 @pytest.mark.asyncio
-async def test_a_token(oid_with_credentials: Tuple[KeycloakOpenID, str, str]):
-    """Test the token method.
+async def test_a_token(oid_with_credentials: tuple[KeycloakOpenID, str, str]) -> None:
+    """
+    Test the token method.
 
     :param oid_with_credentials: Keycloak OpenID client with pre-configured user credentials
     :type oid_with_credentials: Tuple[KeycloakOpenID, str, str]
@@ -631,9 +705,11 @@ async def test_a_token(oid_with_credentials: Tuple[KeycloakOpenID, str, str]):
 
 @pytest.mark.asyncio
 async def test_a_exchange_token(
-    oid_with_credentials: Tuple[KeycloakOpenID, str, str], admin: KeycloakAdmin
-):
-    """Test the exchange token method.
+    oid_with_credentials: tuple[KeycloakOpenID, str, str],
+    admin: KeycloakAdmin,
+) -> None:
+    """
+    Test the exchange token method.
 
     :param oid_with_credentials: Keycloak OpenID client with pre-configured user credentials
     :type oid_with_credentials: Tuple[KeycloakOpenID, str, str]
@@ -652,7 +728,7 @@ async def test_a_exchange_token(
             await admin.a_get_client_role(
                 client_id=admin.get_client_id(client_id="realm-management"),
                 role_name="impersonation",
-            )
+            ),
         ],
     )
 
@@ -669,7 +745,9 @@ async def test_a_exchange_token(
 
     # Exchange token with the new user
     new_token = oid.exchange_token(
-        token=token["access_token"], audience=oid.client_id, subject=username
+        token=token["access_token"],
+        audience=oid.client_id,
+        subject=username,
     )
     assert await oid.a_userinfo(token=new_token["access_token"]) == {
         "email": f"{username}@test.test",
@@ -684,8 +762,9 @@ async def test_a_exchange_token(
 
 
 @pytest.mark.asyncio
-async def test_a_logout(oid_with_credentials):
-    """Test logout.
+async def test_a_logout(oid_with_credentials: tuple[KeycloakOpenID, str, str]) -> None:
+    """
+    Test logout.
 
     :param oid_with_credentials: Keycloak OpenID client with pre-configured user credentials
     :type oid_with_credentials: Tuple[KeycloakOpenID, str, str]
@@ -693,16 +772,17 @@ async def test_a_logout(oid_with_credentials):
     oid, username, password = oid_with_credentials
 
     token = await oid.a_token(username=username, password=password)
-    assert await oid.a_userinfo(token=token["access_token"]) != dict()
-    assert await oid.a_logout(refresh_token=token["refresh_token"]) == dict()
+    assert await oid.a_userinfo(token=token["access_token"]) != {}
+    assert await oid.a_logout(refresh_token=token["refresh_token"]) == {}
 
     with pytest.raises(KeycloakAuthenticationError):
         await oid.a_userinfo(token=token["access_token"])
 
 
 @pytest.mark.asyncio
-async def test_a_certs(oid: KeycloakOpenID):
-    """Test certificates.
+async def test_a_certs(oid: KeycloakOpenID) -> None:
+    """
+    Test certificates.
 
     :param oid: Keycloak OpenID client
     :type oid: KeycloakOpenID
@@ -711,8 +791,9 @@ async def test_a_certs(oid: KeycloakOpenID):
 
 
 @pytest.mark.asyncio
-async def test_a_public_key(oid: KeycloakOpenID):
-    """Test public key.
+async def test_a_public_key(oid: KeycloakOpenID) -> None:
+    """
+    Test public key.
 
     :param oid: Keycloak OpenID client
     :type oid: KeycloakOpenID
@@ -722,9 +803,11 @@ async def test_a_public_key(oid: KeycloakOpenID):
 
 @pytest.mark.asyncio
 async def test_a_entitlement(
-    oid_with_credentials_authz: Tuple[KeycloakOpenID, str, str], admin: KeycloakAdmin
-):
-    """Test entitlement.
+    oid_with_credentials_authz: tuple[KeycloakOpenID, str, str],
+    admin: KeycloakAdmin,
+) -> None:
+    """
+    Test entitlement.
 
     :param oid_with_credentials_authz: Keycloak OpenID client configured as an authorization
         server with client credentials
@@ -735,7 +818,7 @@ async def test_a_entitlement(
     oid, username, password = oid_with_credentials_authz
     token = await oid.a_token(username=username, password=password)
     resource_server_id = admin.get_client_authz_resources(
-        client_id=admin.get_client_id(oid.client_id)
+        client_id=admin.get_client_id(oid.client_id),
     )[0]["_id"]
 
     with pytest.raises(KeycloakDeprecationError):
@@ -743,8 +826,9 @@ async def test_a_entitlement(
 
 
 @pytest.mark.asyncio
-async def test_a_introspect(oid_with_credentials: Tuple[KeycloakOpenID, str, str]):
-    """Test introspect.
+async def test_a_introspect(oid_with_credentials: tuple[KeycloakOpenID, str, str]) -> None:
+    """
+    Test introspect.
 
     :param oid_with_credentials: Keycloak OpenID client with pre-configured user credentials
     :type oid_with_credentials: Tuple[KeycloakOpenID, str, str]
@@ -754,18 +838,22 @@ async def test_a_introspect(oid_with_credentials: Tuple[KeycloakOpenID, str, str
 
     assert (await oid.a_introspect(token=token["access_token"]))["active"]
     assert await oid.a_introspect(
-        token=token["access_token"], rpt="some", token_type_hint="requesting_party_token"
+        token=token["access_token"],
+        rpt="some",
+        token_type_hint="requesting_party_token",  # noqa: S106
     ) == {"active": False}
 
     with pytest.raises(KeycloakRPTNotFound):
         await oid.a_introspect(
-            token=token["access_token"], token_type_hint="requesting_party_token"
+            token=token["access_token"],
+            token_type_hint="requesting_party_token",  # noqa: S106
         )
 
 
 @pytest.mark.asyncio
-async def test_a_decode_token(oid_with_credentials: Tuple[KeycloakOpenID, str, str]):
-    """Test decode token.
+async def test_a_decode_token(oid_with_credentials: tuple[KeycloakOpenID, str, str]) -> None:
+    """
+    Test decode token asynchronously.
 
     :param oid_with_credentials: Keycloak OpenID client with pre-configured user credentials
     :type oid_with_credentials: Tuple[KeycloakOpenID, str, str]
@@ -781,11 +869,59 @@ async def test_a_decode_token(oid_with_credentials: Tuple[KeycloakOpenID, str, s
     assert decoded_refresh_token["typ"] == "Refresh", decoded_refresh_token
 
 
+@pytest.mark.asyncio
+async def test_a_decode_token_invalid_token(
+    oid_with_credentials: tuple[KeycloakOpenID, str, str],
+) -> None:
+    """
+    Test decode token asynchronously an invalid token.
+
+    :param oid_with_credentials: Keycloak OpenID client with pre-configured user credentials
+    :type oid_with_credentials: Tuple[KeycloakOpenID, str, str]
+    """
+    oid, username, password = oid_with_credentials
+    token = await oid.a_token(username=username, password=password)
+    access_token = token["access_token"]
+    decoded_access_token = await oid.a_decode_token(token=access_token)
+
+    key = await oid.a_public_key()
+    key = "-----BEGIN PUBLIC KEY-----\n" + key + "\n-----END PUBLIC KEY-----"
+    key = jwcrypto.jwk.JWK.from_pem(key.encode("utf-8"))
+
+    invalid_access_token = access_token + "a"
+    with pytest.raises(jwcrypto.jws.InvalidJWSSignature):
+        decoded_invalid_access_token = await oid.a_decode_token(
+            token=invalid_access_token,
+            validate=True,
+        )
+
+    with pytest.raises(jwcrypto.jws.InvalidJWSSignature):
+        decoded_invalid_access_token = await oid.a_decode_token(
+            token=invalid_access_token,
+            validate=True,
+            key=key,
+        )
+
+    decoded_invalid_access_token = await oid.a_decode_token(
+        token=invalid_access_token,
+        validate=False,
+    )
+    assert decoded_access_token == decoded_invalid_access_token
+
+    decoded_invalid_access_token = await oid.a_decode_token(
+        token=invalid_access_token,
+        validate=False,
+        key=key,
+    )
+    assert decoded_access_token == decoded_invalid_access_token
+
+
 @pytest.mark.asyncio
 async def test_a_load_authorization_config(
-    oid_with_credentials_authz: Tuple[KeycloakOpenID, str, str]
-):
-    """Test load authorization config.
+    oid_with_credentials_authz: tuple[KeycloakOpenID, str, str],
+) -> None:
+    """
+    Test load authorization config.
 
     :param oid_with_credentials_authz: Keycloak OpenID client configured as an authorization
         server with client credentials
@@ -800,15 +936,18 @@ async def test_a_load_authorization_config(
     assert isinstance(oid.authorization.policies["test-authz-rb-policy"].roles[0], Role)
     assert len(oid.authorization.policies["test-authz-rb-policy"].permissions) == 2
     assert isinstance(
-        oid.authorization.policies["test-authz-rb-policy"].permissions[0], Permission
+        oid.authorization.policies["test-authz-rb-policy"].permissions[0],
+        Permission,
     )
 
 
 @pytest.mark.asyncio
 async def test_a_has_uma_access(
-    oid_with_credentials_authz: Tuple[KeycloakOpenID, str, str], admin: KeycloakAdmin
-):
-    """Test has UMA access.
+    oid_with_credentials_authz: tuple[KeycloakOpenID, str, str],
+    admin: KeycloakAdmin,
+) -> None:
+    """
+    Test has UMA access.
 
     :param oid_with_credentials_authz: Keycloak OpenID client configured as an authorization
         server with client credentials
@@ -825,7 +964,10 @@ async def test_a_has_uma_access(
     )
     assert (
         str(
-            await oid.a_has_uma_access(token=token["access_token"], permissions="Default Resource")
+            await oid.a_has_uma_access(
+                token=token["access_token"],
+                permissions="Default Resource",
+            ),
         )
         == "AuthStatus(is_authorized=True, is_logged_in=True, missing_permissions=set())"
     )
@@ -841,17 +983,19 @@ async def test_a_has_uma_access(
     assert (
         str(
             await oid.a_has_uma_access(
-                token=admin.connection.token["access_token"], permissions="Default Resource"
-            )
+                token=admin.connection.token["access_token"],
+                permissions="Default Resource",
+            ),
         )
         == "AuthStatus(is_authorized=False, is_logged_in=False, missing_permissions="
-        + "{'Default Resource'})"
+        "{'Default Resource'})"
     )
 
 
 @pytest.mark.asyncio
-async def test_a_get_policies(oid_with_credentials_authz: Tuple[KeycloakOpenID, str, str]):
-    """Test get policies.
+async def test_a_get_policies(oid_with_credentials_authz: tuple[KeycloakOpenID, str, str]) -> None:
+    """
+    Test get policies.
 
     :param oid_with_credentials_authz: Keycloak OpenID client configured as an authorization
         server with client credentials
@@ -868,17 +1012,17 @@ async def test_a_get_policies(oid_with_credentials_authz: Tuple[KeycloakOpenID,
 
     orig_client_id = oid.client_id
     oid.client_id = "account"
-    assert await oid.a_get_policies(token=token["access_token"], method_token_info="decode") == []
+    assert await oid.a_get_policies(token=token["access_token"], method_token_info="decode") == []  # noqa: S106
     policy = Policy(name="test", type="role", logic="POSITIVE", decision_strategy="UNANIMOUS")
     policy.add_role(role="account/view-profile")
     oid.authorization.policies["test"] = policy
     assert [
         str(x)
-        for x in await oid.a_get_policies(token=token["access_token"], method_token_info="decode")
+        for x in await oid.a_get_policies(token=token["access_token"], method_token_info="decode")  # noqa: S106
     ] == ["Policy: test (role)"]
     assert [
         repr(x)
-        for x in await oid.a_get_policies(token=token["access_token"], method_token_info="decode")
+        for x in await oid.a_get_policies(token=token["access_token"], method_token_info="decode")  # noqa: S106
     ] == ["<Policy: test (role)>"]
     oid.client_id = orig_client_id
 
@@ -888,8 +1032,11 @@ async def test_a_get_policies(oid_with_credentials_authz: Tuple[KeycloakOpenID,
 
 
 @pytest.mark.asyncio
-async def test_a_get_permissions(oid_with_credentials_authz: Tuple[KeycloakOpenID, str, str]):
-    """Test get policies.
+async def test_a_get_permissions(
+    oid_with_credentials_authz: tuple[KeycloakOpenID, str, str],
+) -> None:
+    """
+    Test get policies.
 
     :param oid_with_credentials_authz: Keycloak OpenID client configured as an authorization
         server with client credentials
@@ -907,26 +1054,31 @@ async def test_a_get_permissions(oid_with_credentials_authz: Tuple[KeycloakOpenI
     orig_client_id = oid.client_id
     oid.client_id = "account"
     assert (
-        await oid.a_get_permissions(token=token["access_token"], method_token_info="decode") == []
+        await oid.a_get_permissions(token=token["access_token"], method_token_info="decode") == []  # noqa: S106
     )
     policy = Policy(name="test", type="role", logic="POSITIVE", decision_strategy="UNANIMOUS")
     policy.add_role(role="account/view-profile")
     policy.add_permission(
         permission=Permission(
-            name="test-perm", type="resource", logic="POSITIVE", decision_strategy="UNANIMOUS"
-        )
+            name="test-perm",
+            type="resource",
+            logic="POSITIVE",
+            decision_strategy="UNANIMOUS",
+        ),
     )
     oid.authorization.policies["test"] = policy
     assert [
         str(x)
         for x in await oid.a_get_permissions(
-            token=token["access_token"], method_token_info="decode"
+            token=token["access_token"],
+            method_token_info="decode",  # noqa: S106
         )
     ] == ["Permission: test-perm (resource)"]
     assert [
         repr(x)
         for x in await oid.a_get_permissions(
-            token=token["access_token"], method_token_info="decode"
+            token=token["access_token"],
+            method_token_info="decode",  # noqa: S106
         )
     ] == ["<Permission: test-perm (resource)>"]
     oid.client_id = orig_client_id
@@ -937,8 +1089,11 @@ async def test_a_get_permissions(oid_with_credentials_authz: Tuple[KeycloakOpenI
 
 
 @pytest.mark.asyncio
-async def test_a_uma_permissions(oid_with_credentials_authz: Tuple[KeycloakOpenID, str, str]):
-    """Test UMA permissions.
+async def test_a_uma_permissions(
+    oid_with_credentials_authz: tuple[KeycloakOpenID, str, str],
+) -> None:
+    """
+    Test UMA permissions.
 
     :param oid_with_credentials_authz: Keycloak OpenID client configured as an authorization
         server with client credentials
@@ -954,8 +1109,9 @@ async def test_a_uma_permissions(oid_with_credentials_authz: Tuple[KeycloakOpenI
 
 
 @pytest.mark.asyncio
-async def test_a_device(oid_with_credentials_device: Tuple[KeycloakOpenID, str, str]):
-    """Test device authorization flow.
+async def test_a_device(oid_with_credentials_device: tuple[KeycloakOpenID, str, str]) -> None:
+    """
+    Test device authorization flow.
 
     :param oid_with_credentials_device: Keycloak OpenID client with pre-configured user
         credentials and device authorization flow enabled
@@ -968,13 +1124,13 @@ async def test_a_device(oid_with_credentials_device: Tuple[KeycloakOpenID, str,
         "user_code": mock.ANY,
         "verification_uri": f"http://localhost:8081/realms/{oid.realm_name}/device",
         "verification_uri_complete": f"http://localhost:8081/realms/{oid.realm_name}/"
-        + f"device?user_code={res['user_code']}",
+        f"device?user_code={res['user_code']}",
         "expires_in": 600,
         "interval": 5,
     }
 
 
-def test_counter_part():
+def test_counter_part() -> None:
     """Test that each function has its async counter part."""
     openid_methods = [
         func for func in dir(KeycloakOpenID) if callable(getattr(KeycloakOpenID, func))
